import datetime
import time

"""
时间日期的相关处理
"""


class XsDateUtils:

    # 获取距离现在时间的任意时间的日期     正数 加,负数 减  return:2019-05-12
    # @staticmethod
    # def getAnyDateTime(day, hour=0, min=0, sec=0):
    #     """
    #          description:  获取距离现在时间的任意时间的日期&时间
    #          day:          天数 1代表当前时间+1天    -1代表当前时间-1天
    #          hour:         小时 2代表当前时间+2h     -2代表当前时间-2h     默认=0
    #          min:          分钟 30代表当前时间+30min -30代表当前时间-30m   默认=0
    #          sec:          秒   120代表当前时间+120s -120代表当前时间-120s 默认=0
    #          return:       2019-05-15 15:37:41 -> str
    #          """
    #
    #     return \
    #         str(datetime.datetime.now() + datetime.timedelta(days=day, hours=hour, minutes=min, seconds=sec)).split(
    #             ".")[0]

    # @staticmethod
    # def getAnyDateSecondTime(day, hour=0, min=0, sec=0):
    #     """
    #          description:  获取距离现在时间的任意时间的秒数
    #          day:          天数 1代表当前时间+1天    -1代表当前时间-1天
    #          hour:         小时 2代表当前时间+2h     -2代表当前时间-2h     默认=0
    #          min:          分钟 30代表当前时间+30min -30代表当前时间-30m   默认=0
    #          sec:          秒   120代表当前时间+120s -120代表当前时间-120s 默认=0
    #          return:       1557902182 -> str
    #          """
    #
    #     anyDay = datetime.datetime.now() + datetime.timedelta(days=day, hours=hour, minutes=min, seconds=sec)
    #     return str(round(time.mktime(anyDay.timetuple())))

    # @staticmethod
    # def getTodayTime():
    #     """
    #          description:  获取当天0点的时间戳
    #          return:       1557676800 -> str
    #          """
    #
    #     return str(round(time.mktime(datetime.date.today().timetuple())))
    #
    # @staticmethod
    # def getCurrentWeekTime():
    #     """
    #          description:  获取本周周一0点
    #          return:       1557676800 -> str
    #          tips:         可替换成: timestamps = time.mktime(time.strptime(time.strftime("%Y-%m-%d", time.localtime(times)), "%Y-%m-%d"))
    #          """
    #
    #     week = int(time.strftime("%w", time.localtime()))
    #     times = round(time.time()) - (week - 1) * 86400
    #     timestamps = time.mktime(datetime.date.fromtimestamp(times).timetuple())
    #     return str(round(timestamps))

    # @staticmethod
    # def getCurrentTimeStr():
    #     """
    #           description:  获取当前时间的可读形式字符串
    #           return:       Mon May 13 11:27:42 2019 -> str
    #           """
    #
    #     return time.ctime()

    # @staticmethod
    # def getCurrentTimeStrByTuple(tupleTime=time.localtime()):
    #     """
    #           description:  获取指定时间的可读形式字符串
    #           tupleTime:    时间元组 可通过time.localtime() or datetime.datetime.now().timetuple()获取 默认当前时间的元组
    #           return:       Mon May 13 11:27:42 2019 -> str
    #           """
    #
    #     return time.asctime(tupleTime)



    @staticmethod
    def get_milli_second(times=None):
        """
        获取指定时间的时间戳-毫秒级
        :param times: 指定时间，默认为当前时间
        :return: 1639728906765 - int
        """
        if not times:
            times = time.time()
        timestamps = round(times * 1000)
        return timestamps

    @staticmethod
    def get_second(times=None):
        """
        获取指定时间的时间戳-秒级
        :param times:指定时间，默认为当前时间
        :return: 1557730377 -> int
        """
        if not times:
            times = time.time()
        timestamps = round(times)
        return timestamps



    @staticmethod
    def add_date(day_num, target=None, tp='d'):
        """
        给定日期，延长或减少指定时间（天数,小时，分钟，秒)
        :param tp: 类型,d代表天，h代表小时，m代表分钟，s代表秒，默认为d
        :param day_num: 指定天数，可以是负数
        :param target: 指定日期，默认为当天
        :return:
        """
        if not target:
            target = datetime.datetime.now()
        if tp == 'd':
            target += datetime.timedelta(days=day_num)
        elif tp == 'h':
            target += datetime.timedelta(hours=day_num)
        elif tp == 'm':
            target += datetime.timedelta(minutes=day_num)
        elif tp == 's':
            target += datetime.timedelta(seconds=day_num)

        return target

    @staticmethod
    def add_days(day_num, target=None):
        return XsDateUtils.add_date(day_num, target, 'd')

    @staticmethod
    def add_hours(day_num, target=None):
        return XsDateUtils.add_date(day_num, target, 's')

    @staticmethod
    def add_minutes(day_num, target=None):
        return XsDateUtils.add_date(day_num, target, 'm')

    @staticmethod
    def get_span(start, end, tp='d'):
        """
        返回两个时间的时间隔
        :param start: 开始时间
        :param end: 结束时间
        :param tp: 类型，d为天数，s为秒数，mic为毫秒数
        :return:
        """
        span = 0
        if tp == 'd':
            span = (end - start).days
        elif tp == 's':
            span = (end - start).seconds
        elif tp == 'mic':
            span = (end - start).microseconds
        return span

    @staticmethod
    def get_span_days(start, end):
        return XsDateUtils.get_span(start, end, tp='d')

    @staticmethod
    def get_span_seconds(start, end):
        return XsDateUtils.get_span(start, end, tp='s')

    @staticmethod
    def get_now():
        """
        获取当前日期及时间点
        :return:
        """
        return datetime.datetime.now()

    @staticmethod
    def seconds_to_str(times=None, format_t="%Y-%m-%d %H:%M:%S"):
        """
              description:  将指定时间戳转换成可视式时间
              times:        指定时间戳，秒级
              return:       2019-05-13 15:00:47 -> str
              tips:         %y 两位数的年份表示（00-99）    %Y 四位数的年份表示（000-9999）   %m 月份（01-12）    %d 月内中的一天（0-31）
                            %H 24小时制小时数（0-23）      %I 12小时制小时数（01-12）        %M 分钟数（00=59）  %S 秒（00-59）   %w 星期（0-6）
         """
        if not times:
            times = time.time()
        timestamps = time.strftime(format_t, time.localtime(times))
        return timestamps

    @staticmethod
    def seconds_to_day_str(times=None):
        """
        将时间戳转换成天的字符串，不包括时间
        :param times:秒 默认当前时间
        :return: return:       2019-05-13 -> str  一天86400秒
        """
        if not times:
            times = time.time()
        timestamps = str(datetime.date.fromtimestamp(round(times)))
        return timestamps

    @staticmethod
    def seconds_to_tuple(times=None):
        """
        将指定时间戳转换成时间元祖
        :param times: 默认当前时间 可传second
        :return: (tm_year=2019, tm_mon=5, tm_mday=13, tm_hour=10, tm_min=9, tm_sec=18, tm_wday=0, tm_yday=133, tm_isdst=0) -> tuple
        """

        if not times:
            times = time.time()
        timestamps = time.localtime(times)
        return timestamps



    @staticmethod
    def tuple_to_seconds(tupleTime=None):
        """
        将指定的时间元祖转换成时间戳
        :param tupleTime:  默认当前时间的元组 可通过time.localtime() or datetime.datetime.now().timetuple()获取
        :return: 1557733061 -> str
        """
        if not tupleTime:
            tupleTime = time.localtime()
        timestamps = str(round(time.mktime(tupleTime)))
        return timestamps

    @staticmethod
    def str_to_tuple(time_str=None,format_t="%Y-%m-%d %H:%M:%S"):
        """
        将时间字符串转换成时间元祖
        :param time_str: 格式化字符串   如:2019-05-13 15:00:47    默认当前时间
        :param format_t: 格式化规则    如:%Y-%m-%d %H:%M:%S      默认%Y-%m-%d %H:%M:%S
        :return: (tm_year=2019, tm_mon=5, tm_mday=13, tm_hour=10, tm_min=9, tm_sec=18, tm_wday=0, tm_yday=133, tm_isdst=0) -> tuple
        """
        if not time_str:
            time_str = str(datetime.datetime.now()).split(".")[0]
        return time.strptime(time_str, format_t)

    @staticmethod
    def get_now_str():
        """
        获取当前时间日期的字符串
        :return: 2019-05-13 14:41:15 -> str
        """
        timestamps = str(datetime.datetime.now()).split(".")[0]
        return timestamps

    @staticmethod
    def get_today_str():
        """
        获取当天的字符串格式
        :return: 2019-05-13 -> str  与 datetime.datetime.now().date()有相同效果
        """

        timestamps = str(datetime.date.today())
        return timestamps

    @staticmethod
    def get_time_to_str():
        """
        获取当前时间的字符串格式
        :return: 14:41:15 -> str
        """
        timestamps = str(datetime.datetime.now().time()).split(".")[0]
        return timestamps




import base64


class XsBase64:

    @staticmethod
    def encodeStr(content: str) -> str:
        return base64.b64encode(content.encode()).decode()

    @staticmethod
    def decodeStr(content: str) -> str:
        return base64.b64decode(content.encode()).decode()

    @staticmethod
    def encodeImage(filepath: str) -> str:
        with open(filepath, "rb") as f:  # 转为二进制格式
            base64_data = base64.b64encode(f.read())  # 使用base64进行加密
        return base64_data.decode()

    @staticmethod
    def decodeImage(save_path: str, content: str) -> str:
        image = base64.b64decode(content.encode())
        with open(save_path, "wb") as f:
            f.write(image)





from unittest import TestCase

from XsCore import XsFso
from XsCore.XsFso.FileList import XsFileList


class FileListTest(TestCase):

    def testReadCode(self):
        content = XsFso.getFileCode(r"D:\web\testddd.txt")
        print(content)

    def testReadAny(self):
        content = XsFso.readFileAnyCode(r"D:\web\testddd.txt")
        print(content)

    def testWriteAny(self):
        XsFso.writeFileAnyCode(r"D:\web\testddd.txt", "ffffffffff举例：有一个data.txt文件，我们不知道它的编码格式，现在我们需要读取文件的编码格式：fff")



import os
from configparser import ConfigParser

"""
操作 ini的工具类
两种实例:
    1.你可以采用 XsIniUtils 来实例化ini对象，构造函数的filename可以是相对路径也可以是绝对路径
        参照测试 testGetAllSections
    2.你可以将你的配置文件放在当前目录下的conf/conf.ini目录下，直接使用xsIni这个单例来操作(推荐)
        参照测试 testDefaultInstance
具体操作请参照测试样例
"""


class XsIniUtils(ConfigParser):
    __default_section = "App"

    def __init__(self, filename):
        """
        构造方法
        :param filename: filename可以是绝对路径或相对路径
        """
        # 调用父类的init方法
        super().__init__()
        # currentPath = os.path.abspath(filename)  # [path, os.path.abspath(path)][isRelative]
        # print(currentPath)  # cfg.ini的路径
        self._filename = filename
        self.read(filename, encoding="utf8")  # python3 如果文件里有中文，需要加上这个encoding

    def getAllSections(self):
        """
        # 获取所有的section
        :return:
        """
        return self.sections()

    def getAppValue(self, keyName: str):
        """
        基于App节点的获取操作
        :param keyName:键名
        :return:返回值字符
        """
        return self.get(self.__default_section, keyName)

    def getAppInt(self, keyName: str):
        """
        基于App节点的获取操作
        :param keyName:键名
        :return:返回值整数
        """
        return self.getint(self.__default_section, keyName)

    def getAppFloat(self, keyName: str):
        """
        基于App节点的获取操作
        :param keyName:键名
        :return:返回浮点
        """
        return self.getfloat(self.__default_section, keyName)

    def getAppBool(self, keyName: str):
        """
        基于App节点的获取操作
        :param keyName:键名
        :return:返回布尔
        """
        return self.getboolean(self.__default_section, keyName)

    def setAppItem(self, key, value):
        """
        基于App节点的更新或添加操作，如果存在key则更新，不存在key则更新
        :param key:键名
        :param value:值
        :return:
        """
        self.set(self.__default_section, key, value)
        with open(self._filename, "w", encoding="utf-8") as fs:
            self.write(fs)


"""
XsIniUtils的默认单例,要求要将配置文件存放在当前目录的conf/conf.ini下
"""
xsIni = XsIniUtils("conf/conf.ini")



import tkinter
from tkinter import filedialog

root = tkinter.Tk()
root.withdraw()


def showInfo(msg: str, title='提示'):
    tkinter.messagebox.showinfo(title, msg)


def showWarning(msg: str, title='警告'):
    tkinter.messagebox.showwarning(title, msg)


def showError(msg: str, title='出错了'):
    tkinter.messagebox.showerror(title, msg)


def askAskYesNo(msg: str, title='提示') -> bool:
    return tkinter.messagebox.askyesno(title, msg)


def askAskYesNoCancel(msg: str, title='提示') -> bool:
    return tkinter.messagebox.askyesnocancel(title, msg)


def askOpenfile():
    fileName = filedialog.askopenfilename()
    return fileName


def askOpenFolder():
    folderPath = filedialog.askdirectory()
    return folderPath



import base64

from Crypto.Cipher import AES
"""
AES对称加密，本工具主要实现CBC模式加密,特点：
1.支持nopadding与非nopadding
2.支持输出base64或hex
"""

def aesEncryptXs(content=None, key=None, vi='aUaCoyTaut13JoQ9', is_base64=True, nopadding=True):
    """
    # 密钥（key）, 密斯偏移量（iv） CBC模式加密
    :param content:
    :param key:
    :param vi:
    :param is_base64:
    :param nopadding:
    :return:
    """
    length = 16  # 这里只是用于下面取余前面别以为是配置
    count = len(content.encode('utf-8'))  # 这是我上传的主要目的，字符长度不同所以不能直接用，需要先编码转成字节
    if (count % length != 0):
        add = length - (count % length)
    else:
        add = 0  # 看看你们对接是满16的时候加上16还是0.这里注意
    # 其它语言nopadding时，python还是需要‘’或'x00'这里注意与其它语言对接注
    # 需要使用上面的字符集去填充，如果是真的是 nopadding 的话 这个加的是
    if nopadding:
        text1 = content + '' + (' ' * add)
    else:
        # 字符串补位--最后一位就是位数 需要补充多少位的数据
        consult = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F"]
        text1 = content + '' + (consult[add - 1] * add)

    cipher = AES.new(key.encode('utf8'), AES.MODE_CBC, vi.encode('utf8'))
    encryptedbytes = cipher.encrypt(text1.encode('utf8'))
    if is_base64:
        return str(base64.b64encode(encryptedbytes), encoding='utf-8')
    else:
        return encryptedbytes.hex().upper()



def aesDecryptXs(content: str, key: str, vi='aUaCoyTaut13JoQ9', is_base64=True, nopadding=True):
    # vi = '5928772605893626'
    # encodebytes = base64.decodebytes(data)

    # 将加密数据转换位bytes类型数据
    cipher = AES.new(key.encode('utf8'), AES.MODE_CBC, vi.encode('utf8'))
    #  # 将加密数据转换位bytes类型数据fromhex
    temp = None
    if is_base64:
        temp = base64.b64decode(content)
    else:
        temp = bytes.fromhex(content.lower())

    text_decrypted = cipher.decrypt(temp)
    # 解密-bytesToString 字节转字符串
    str_text_decrypted = bytes.decode(text_decrypted, encoding='utf8')
    if nopadding:
        return str_text_decrypted.rstrip(' ')
    else:
        consult = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F"]
        # 截取最后一位标记- # 字符串补位--最后一位就是位数 需要补充多少位的数据
        num = 0
        # 获取最后一个位数的
        mark_str_index = str_text_decrypted[-1:]
        for i, val in enumerate(consult):
            # 计算出现的位置索引号
            if val == mark_str_index:
                num = int(i) + 1
                break
    return str_text_decrypted[:-num]


from .HttpUtils import *
from .XsDateUtils import *
from .RegexUtils import *
from .XsStringVal import *
from .IniUtils import *
from .HttpUtils import *
from XsCore.XsBase64 import *
from .Encrypt import *
from .Logger import *
from .EmailSender import *
from .DingRobot import *
from .XsJob import *
from .XsPool import *
from .XsDialogs import *



from typing import Any

from XsCore import XsRegexUtils


class XsStringVal:

    @staticmethod
    def is_string(obj: Any) -> bool:
        """
        判断对象是否为字符串
        :param obj:
        :return:
        """
        return isinstance(obj, str)

    @staticmethod
    def is_empty(input_string: str) -> bool:
        """
        检查字符串是否为空（它必须至少包含一个非空格字符）
        *Examples:*

        >>> is_full_string(None) # returns false
        >>> is_full_string('') # returns false
        >>> is_full_string(' ') # returns false
        >>> is_full_string('hello') # returns true

        :param input_string: String to check.
        :type input_string: str
        :return: True if not empty, false otherwise.
        """
        if input_string:
            return input_string.strip() == ''
        return False

    @staticmethod
    def is_number(input_string: str) -> bool:
        """
        判断字符是否为数字
        :param input_string:
        :return:
        """
        try:
            float(input_string)
            return True
        except ValueError:
            pass

        try:
            import unicodedata
            unicodedata.numeric(input_string)
            return True
        except (TypeError, ValueError):
            pass

        return False

    @staticmethod
    def is_integer(input_string: str) -> bool:
        """
        判断字符是否为整数
        >>> is_integer('42') # returns true
        >>> is_integer('42.0') # returns false

        :param input_string: String to check
        :type input_string: str
        :return: True if integer, false otherwise
        """
        return XsStringVal.is_number(input_string) and '.' not in input_string

    @staticmethod
    def is_decimal(input_string: str) -> bool:
        """
         判断字符是否为decimal
        >>> is_decimal('42.0') # returns true
        >>> is_decimal('42') # returns false

        :param input_string: String to check
        :type input_string: str
        :return: True if integer, false otherwise
        """
        return XsStringVal.is_number(input_string) and '.' in input_string

    @staticmethod
    def is_email(txt: str) -> bool:
        return XsRegexUtils.IsMatch(r'[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(?:\.[a-zA-Z0-9_-]+)', txt)

    @staticmethod
    def is_id_card(txt: str) -> bool:
        return XsRegexUtils.IsMatch(
            r'[1-9]\d{5}(18|19|([23]\d))\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\d{3}[0-9Xx]', txt)

    @staticmethod
    def is_mobile(txt: str) -> bool:
        return XsRegexUtils.IsMatch(r'1(3|4|5|6|7|8|9)\d{9}', txt)

    @staticmethod
    def is_tel(txt: str) -> bool:
        return XsRegexUtils.IsMatch(r'\d{3}-\d{8}|\d{4}-\d{7}', txt)

    @staticmethod
    def is_ip(txt: str) -> bool:
        return XsRegexUtils.IsMatch(r'((?:(?:25[0-5]|2[0-4]\d|[01]?\d?\d)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d?\d))', txt)

    @staticmethod
    def is_post_code(txt: str) -> bool:
        return XsRegexUtils.IsMatch(r'[1-9]\d{5}(?!\d)', txt)

    @staticmethod
    def is_url(txt: str) -> bool:
        return XsRegexUtils.IsMatch(r'[a-zA-z]+://[^\s]*', txt)


from unittest import TestCase

from XsCore import XsPath


class FileListTest(TestCase):

    def testGetSubFiles(self):
        fs = XsPath.getSubFiles(r"D:\web",['.txt'])
        print(fs)

    def testGetSubFiles2(self):
        fs = XsPath.getSubFiles("images")
        print(fs)



import json

import requests

"""
钉钉地群机器人消息发送
"""


class DingRobot:
    @staticmethod
    def send(access_token: str, msg: str, is_at_all=True):
        """
        发送群消息
        :param access_token: 申请到的机器人token
        :param msg: 要发送的消息,注:必须的你设置在机器人里的关键字，如配置有关键字【行情】，可以这样编写【行情提醒: BTC: 流通市值：6.97万亿 价格：37.01万】
        :param is_at_all: 是否@所有人
        :return:
        """
        # 发起POST请求时，必须将字符集编码设置成UTF - 8。
        # 每个机器人每分钟最多发送20条。消息发送太频繁会严重影响群成员的使用体验，大量发消息的场景(譬如系统监控报警)
        # 可以将这些信息进行整合，通过markdown消息以摘要的形式发送到群里。
        # 请求的URL，WebHook地址
        webhook = f"https://api.dingtalk.com/robot/send?access_token={access_token}"
        # 构建请求头部
        header = {
            "Content-Type": "application/json",
            "Charset": "UTF-8"
        }
        # 构建请求数据
        message = {

            "msgtype": "text",
            "text": {
                "content": msg
            },
            "at": {

                "isAtAll": is_at_all
            }

        }
        # 对请求的数据进行json封装
        message_json = json.dumps(message)
        # 发送请求
        info = requests.post(url=webhook, data=message_json, headers=header)
        # 打印返回的结果
        return info.text



from datetime import date

from apscheduler.schedulers.blocking import BlockingScheduler


class XsJob(BlockingScheduler):
    # def __init__(self):
    #     pass

    def addJob(self, datetime, fn, fn_args=[]):
        self.add_job(fn, 'date', run_date=datetime, args=fn_args)

    def addTimer_Seconds(self, sds: int, fn, fn_args=None):
        self.add_job(fn, 'interval', seconds=sds, args=fn_args)

    def addTimer_Minutes(self, mins: int, fn, fn_args: []):
        self.add_job(fn, 'interval', minutes=mins, args=fn_args)




from unittest import TestCase

from XsCore import Logger


class LoggerTest(TestCase):

    def testLog(self):
        log = Logger("logs/xs_core.log")
        indexs = 0
        while indexs < 1000:
            log.info(f"fadfadf{indexs}")
            indexs += 1




"""
aes采用pycryptodome库：
pip install pycryptodome
"""
from XsCore.Encrypts import xsMd5, aesEncryptXs, aesDecryptXs,   rsa_create_keys, \
    rsa_encrypt_by_public_key,  rsa_sign_by_private_key, rsa_decrypt_by_private_key, \
    rsa_check_sign_by_public_key


class Encrypt:
    @staticmethod
    def md5(content: str):
        return xsMd5(content)

    # 密钥（key）, 密斯偏移量（iv） CBC模式加密
    @staticmethod
    def aesEncrypt(content=None, key=None, vi='aUaCoyTaut13JoQ9', is_base64=True, nopadding=True):
        return aesEncryptXs(content, key, vi, is_base64, nopadding)


    @staticmethod
    def aesDecrypt(content: str, key: str, vi='aUaCoyTaut13JoQ9', is_base64=True, nopadding=True):
        return aesDecryptXs(content, key, vi, is_base64, nopadding)



    @staticmethod
    def rsaCreateKey():
        return rsa_create_keys()

    @staticmethod
    def rsaEncryptByPublic(message, public_key: bytes):
        """
        使用公钥加密
        :param message:要加密的内容
        :param public_key:公钥
        :return:返回加密结果->bytes,如 b'C+zWS1NFw7sC8...
        """
        return rsa_encrypt_by_public_key(message, public_key)

    @staticmethod
    def rsaDecryptByPrivate(message, private_key: bytes):
        """
        使用私钥解密
        :param message:要解密的内容
        :param private_key:私钥
        :return:返回明文
        """
        return rsa_decrypt_by_private_key(message, private_key)

    @staticmethod
    def rsaSignByPrivate(message, private_key: bytes):
        """
        使用私钥对内容进行签名
        :param message: 要签名的内容
        :param private_key: 私钥
        :return: 返回签名结果->bytes,如 b'dlJ1lLJB60...
        """
        return rsa_sign_by_private_key(message, private_key)

    def rsaCheckSignByPublic(message, signature, public_key: bytes) -> bool:
        """
        使用公钥验证签名数据是否正确
        :param message: 被签名的原文
        :param signature: 签名数据
        :param public_key: 公钥
        :return: 返回是否正确，bool
        """
        return rsa_check_sign_by_public_key(message, signature, public_key)


'''
RSA 非对称加密:
加密的 内容 最大长度是 证书key位数/8 - 11
1024 bit的证书，被加密的最长 1024/8 - 11=117
2048 bit的证书，被加密的最长 2048/8 - 11 =245
RSA 非对称加密主要实现以下4个功能:
1.公钥加密
2.私钥解密
3.私钥签名
4.公钥验签
'''
import base64

from Crypto import Random
from Crypto.Hash import SHA
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5 as Cipher_pkcs1_v1_5
from Crypto.Signature import PKCS1_v1_5 as Signature_pkcs1_v1_5

encode_gbk_utf8 = 'utf-8'  # 全局编码方式 utf-8 | gbk
key_num = 1024  # 证书key位数


def rsa_create_keys() -> {}:
    """
    RSA秘钥对的生成
    :return: 以字典的方式，返回私钥与公钥
    """
    random_generator = Random.new().read  # 伪随机数生成器
    rsa = RSA.generate(key_num, random_generator)  # rsa算法生成实例
    # 秘钥对的生成
    private_pem = rsa.exportKey()
    public_pem = rsa.publickey().exportKey()
    return {"private": private_pem, "public": public_pem}


def rsa_encrypt_by_public_key(message, public_key: bytes):
    """
    使用公钥加密
    :param message:要加密的内容
    :param public_key:公钥
    :return:返回加密结果->bytes,如 b'C+zWS1NFw7sC8...
    """
    rsakey = RSA.importKey(public_key)  # 导入读取到的公钥
    cipher = Cipher_pkcs1_v1_5.new(rsakey)  # 生成对象
    # 加密message明文，python3加密的数据必须是bytes，不能是str
    cipher_text = base64.b64encode(cipher.encrypt(
        message.encode(encoding=encode_gbk_utf8)))
    return cipher_text


def rsa_decrypt_by_private_key(message, private_key: bytes):
    """
    使用私钥解密
    :param message:要解密的内容
    :param private_key:私钥
    :return:返回明文
    """
    rsakey = RSA.importKey(private_key)  # 导入读取到的私钥
    cipher = Cipher_pkcs1_v1_5.new(rsakey)  # 生成对象
    # 将密文解密成明文，返回的是bytes类型，需要自己转成str,主要是对中文的处理
    text = cipher.decrypt(base64.b64decode(message), "ERROR")
    return text.decode(encoding=encode_gbk_utf8)


def rsa_sign_by_private_key(message, private_key: bytes):
    """
    使用私钥对内容进行签名
    :param message: 要签名的内容
    :param private_key: 私钥
    :return: 返回签名结果->bytes,如 b'dlJ1lLJB60...
    """
    rsakey = RSA.importKey(private_key)
    signer = Signature_pkcs1_v1_5.new(rsakey)
    digest = SHA.new()
    digest.update(message.encode(encoding=encode_gbk_utf8))
    sign = signer.sign(digest)
    signature = base64.b64encode(sign)  # 对结果进行base64编码
    return signature


def rsa_check_sign_by_public_key(message, signature, public_key: bytes) -> bool:
    """
    使用公钥验证签名数据是否正确
    :param message: 被签名的原文
    :param signature: 签名数据
    :param public_key: 公钥
    :return: 返回是否正确，bool
    """
    rsakey = RSA.importKey(public_key)
    verifier = Signature_pkcs1_v1_5.new(rsakey)
    digest = SHA.new()
    # 注意内容编码和base64解码问题
    digest.update(message.encode(encoding=encode_gbk_utf8))
    is_verify = verifier.verify(digest, base64.b64decode(signature))
    return is_verify



import concurrent.futures
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor

from XsCore.XsException import XsException


class XsPool:
    def __init__(self, max_task=10,is_future=False, is_process=False):
        """
        构造一个线程或进程池实例
        :param max_task: 最大并发数
        :param is_future: 是否获取任务的使用情况
        :param is_process: 是否进程池，默认为线程池，线程池只能使用一个cpu，如果为True，将使用进程池
        """
        self.executor = ProcessPoolExecutor(max_workers=max_task) if is_process else ThreadPoolExecutor(max_workers=max_task)
        self.tasks = []
        self.isfuture = is_future

    def addTask(self, fn, /, *args, **kwargs):
        """
        添加一个任务到池子
        :param fn: 任务，方法或者说是函数
        :param args: 函数的参数，或多个
        :param kwargs: 函数的参数，或多个，关键字的方式
        :return:
        """
        # future列表中每个future完成的顺序，和它在列表中的顺序并不一定完全一致。
        # 到底哪个先完成、哪个后完成，取决于系统的调度和每个future的执行时间

        future = self.executor.submit(fn, *args, **kwargs)
        if self.isfuture:
            self.tasks.append(future)

    def __clearDone(self):
        """
        清理已经完成的任务
        :return:
        """
        rmTem = []
        for task in self.tasks:
            if task.done():  # done()返回True表示任务成功取消或者完成
                rmTem.append(task)
        for tp in rmTem:
            self.tasks.remove(tp)

    def getWaitingCount(self):
        """
        获取等待中的任务数
        :return:
        """
        if self.isfuture:
            self.__clearDone()
            return len(self.tasks)
        else:
            raise XsException(err="要调用此方法，请在构造的时候指定is_future为True")

    def getRunningCount(self):
        """
        获取执行中的任务数
        :return:
        """
        if self.isfuture:
            _count = 0
            for task in self.tasks:
                if task.running():
                    _count += 1
            return _count
        else:
            raise XsException(err="要调用此方法，请在构造的时候指定is_future为True")


    def shutdown(self):
        self.executor.shutdown()



# from typing import Generic, TypeVar

from sqlalchemy import create_engine, select, exists, func
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm import Session
from XsCore.db import Base, db_provider

# T = TypeVar("T")
from tests.db.UserEntity import UserEntity


class DbDal:

    def __init__(self, MT):
        self.engine = db_provider.engine
        self.__MT = MT

    def add(self, model: Base):
        with Session(self.engine) as session:
            session.add(model)
            session.commit()

    def addList(self, models: list):
        with Session(self.engine) as session:
            session.add_all(models)
            session.commit()

    def getOne(self, data_id):
        with Session(self.engine) as session:
            return session.query(self.__MT).filter_by(id=data_id).first()

    def getOneWhere(self, **where):
        with Session(self.engine) as session:
            return session.query(self.__MT).filter_by(**where).first()

    def getAll(self, *orderBy):
        with Session(self.engine) as session:
            return session.query(self.__MT).order_by(*orderBy)

    def getList(self, where, orderBy=None):
        with Session(self.engine) as session:
            return session.query(self.__MT).filter(where).order_by(orderBy)

    def getTop(self, num):
        with Session(self.engine) as session:
            return session.query(self.__MT).order_by(self.__MT.id.desc()).limit(num)

    def getPageWhere(self, page_index, page_size, where, orderBy=None):
        with Session(self.engine) as session:
            return session.query(self.__MT).filter(where).order_by(orderBy).limit(page_size).offset(
                (page_index - 1) * page_size)

    def getPageAll(self, page_index, page_size, orderBy=None):
        with Session(self.engine) as session:
            return session.query(self.__MT).order_by(orderBy).limit(page_size).offset(
                (page_index - 1) * page_size)

    def existsId(self, data_id):
        with Session(self.engine) as session:
            return session.query(exists().where(self.__MT.id == data_id)).scalar()

    def exists(self, *where):
        with Session(self.engine) as session:
            return session.query(exists().where(*where)).scalar()

    def getMaxId(self):
        with Session(self.engine) as session:
            return session.query(func.max(self.__MT.id)).scalar()

    def getCountAll(self):
        with Session(self.engine) as session:
            return session.query(func.count(self.__MT.id)).scalar()

    def getCountWhere(self, *where):
        with Session(self.engine) as session:
            return session.query(func.count(self.__MT.id)).filter(*where).scalar()

    def delId(self, data_id):
        with Session(self.engine) as session:
            session.query(self.__MT).filter_by(id=data_id).delete()
            session.commit()

    def delWhere(self, **where):
        with Session(self.engine) as session:
            session.query(self.__MT).filter_by(**where).delete()
            session.commit()

    def delAll(self):
        with Session(self.engine) as session:
            session.query(self.__MT).delete()
            session.commit()



from unittest import TestCase

from XsCore import XsIniUtils, xsIni


class StringUtilsTest(TestCase):

    def testGetAllSections(self):
        """
        演示操作指定目录下的ini文件
        :return:
        """
        cf = XsIniUtils("conf/conf.ini")
        print("siteName:", cf.getAppValue('sitename'))

        cf.setAppItem("sitename3", "binanceusdm,aofex")

    def testDefaultInstance(self):
        """
        这里演示使用默认的实例操作ini,要求一定要将ini文件保存在当前目录下的conf/conf.ini
        :return:
        """
        print("siteName2:", xsIni.getAppValue('sitename2'))



from unittest import TestCase

from XsCore import XsHttp, XsStrings


class StringUtilsTest(TestCase):

    def testGetHtmlUrls(self):
        htmlc = XsHttp.getHtml('http://www.beimai.com/baike/243a107743b0c.html')
        urls = XsStrings.getHtmlUrls(htmlc)
        for url in urls:
            print(url)

    def testGetChinese(self):
        cns = XsStrings.getChinese("fsfs我是，opopl中国人")
        for cn in cns:
            print(cn)

    def testSplitLines(self):
        rzs = XsStrings.splitLines("ab c\n\nde fg\rkl\r\n")
        print(rzs)

    def testSplit(self):
        rzs = XsStrings.split("ab cnkde fgnkl",",")
        print(rzs)

    def testCutStartEnd(self):
        testTxt = 'Cats are aaa smarter than bbb dogs'
        rz = XsStrings.cutStartEnd(testTxt, "aaa", "bbb")
        print(rz)


import re
from unittest import TestCase

from XsCore import XsRegexUtils, XsHttp


class FileListTest(TestCase):
    def testFindStrToList(self):
        htmlc = XsHttp.getText('http://www.beimai.com/baike/243a107743b0c.html')

        rzs = XsRegexUtils.FindStrToList(
            r'(?:(?:http:\/\/)|(?:https:\/\/))?(?:[\w](?:[\w\-]{0,61}[\w])?\.)+[a-zA-Z]{2,6}(?:\/)', htmlc)
        for url in rzs:
            print(url)

    def testFindStrToList(self):
        htmlc = XsHttp.getText('http://www.beimai.com/baike/243a107743b0c.html')

        rzs = XsRegexUtils.FindStrToList(r'[\u4e00-\u9fa5]+', htmlc)
        for url in rzs:
            print(url)

    def testFindOne(self):
        testTxt = 'Cats are aaa smarter than bbb dogs'

        rzs = XsRegexUtils.FindOne(r'(?<=(aaa))[.\s\S]*?(?=(bbb))', testTxt)

        print("结果：", rzs)

    def testReplaceAll(self):
        phone = "2004-959-559 # 这是一个国外电话号码<image src='https://www.runoob.com/python/python-reg-expressions.html' />"

        rzs = XsRegexUtils.ReplaceAll(r'[<].*?[>]', phone, "")
        print(rzs)

    def testReplaceCustom(self):
        testTxt = 'A23G4HFD567'
        rzs = XsRegexUtils.ReplaceCustom(r'(?P<value>\d+)', testTxt, self.__customReplace)
        print(rzs)

    def __customReplace(self, matched):
        value = int(matched.group('value'))
        return str(value * 2)

    def testIsMatch(self):
        isHave = XsRegexUtils.IsMatch(r"\d{15}|\d{18}", "44082419800918186655588444")
        self.assertTrue(isHave, True)



"""
@File       :   setup.py
@Author     :   cqs
@Time       :   2021-10-9 17:22
@Version    :   1.0
@Contact    :   cqs363@163.com
@Detect       :   None
"""

from setuptools import setup, find_packages  # 这个包没有可以pip一下

setup(
    name="xsCorePy",  # 这个是pip项目发布的名称
    version="1.0.3",  # 版本号，pip默认安装最新版
    keywords=("utils", "xsCorePy"),
    description="模块描述",
    long_description="1.添加日期工具类 2.添加json工具类",
    license="MIT Licence",

    url="https://github.com/yibo7/xsCorePy",  # 项目相关文件地址，一般是github，有没有都行吧
    author="cqs263",
    author_email="cqs363@163.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=["configparser","requests","pycryptodome","apscheduler","sqlalchemy","chardet"]  # 该模块需要的第三方库
)



import logging
from logging.handlers import RotatingFileHandler

from XsCore.XsFso import XsPath


class Logger:
    def __init__(self, path, log_count=100, max_mb=1, cmd_level=logging.DEBUG, file_level=logging.DEBUG):
        """
        创建一个日志操作实例
        :param path: 日志的路径，可以是相对路径，也可以是绝对路径
        :param log_count: 最多保留几个日志文件，默认最多保留100个日志文件
        :param max_mb:  一个日志文件最大保存多少MB
        :param cmd_level: 终端输出的日志级别
        :param file_level:  文件输出日志级别
        """

        # # <editor-fold desc="判断日志文件的目录是否存在，如果不存在就创建一个">
        if not XsPath.isFullPath(path):
            path = XsPath.getFullPath(path)

        XsPath.exists(path, isMake=True)
        # </editor-fold>

        self.logger = logging.getLogger(path)
        self.logger.setLevel(logging.DEBUG)
        fmt = logging.Formatter('[%(asctime)s] [%(levelname)s] %(message)s', '%Y-%m-%d %H:%M:%S')
        # 设置CMD日志
        sh = logging.StreamHandler()
        sh.setFormatter(fmt)
        sh.setLevel(cmd_level)
        self.logger.addHandler(sh)

        # 设置文件日志
        # maxBytes = 1 * 1024 * 1024
        file_hd = RotatingFileHandler(filename=path, maxBytes=max_mb * 1024 * 1024, backupCount=log_count,
                                      encoding='utf-8')
        file_hd.setFormatter(fmt)
        file_hd.setLevel(file_level)
        self.logger.addHandler(file_hd)

    def debug(self, message):
        self.logger.debug(message)

    def info(self, message):
        self.logger.info(message)

    def war(self, message):
        self.logger.warn(message)

    def error(self, message):
        self.logger.error(message)

    def cri(self, message):
        self.logger.critical(message)





from enum import Enum

from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base, sessionmaker, scoped_session


class DataBaseType(Enum):
    sqlite = 1
    mysql = 2

Base = declarative_base()
class DbProvider:
    def __init__(self,dbtype: DataBaseType):
        self.engine = create_engine('sqlite:///db.db?check_same_thread=False', echo=True)

        # 创建Session类实例
        db_session = scoped_session(sessionmaker(autocommit=False,
                                                 autoflush=False,
                                                 bind=self.engine))


        # Session = sessionmaker(bind=self.engine)
        self.session = db_session

        # Base.query = db_session.query_property()

    def create_all_table(self):
        Base.metadata.create_all(self.engine)  # 创建表结构

db_provider = DbProvider(DataBaseType.sqlite)



import json

from XsCore import XsFso


class XsJson:
    def __init__(self, path):
        """
        :param path: json文件的路径，可以是相对路径 'data/action.json'
        """
        self.save_path = path

    def save(self, jsonobj):
        """
        将对象序列化到json文件
        :param jsonobj: 可以是列表，也可以是字典
        :return:
        """
        with open(self.save_path, 'w') as fp:
            json.dump(jsonobj, fp, indent=4)

    def load(self):
        str_json = XsFso.readFile(self.save_path)
        if len(str_json) > 1:
            return json.loads(str_json)
        return None



from unittest import TestCase

from XsCore import EmailSender

smtpServer = 'smtp.163.com'
userName = 'xxx@163.com'
passWord = 'xxxxxx'

class EmailSenderTest(TestCase):

    def testSendEmailText(self):
        em_obj = EmailSender(smtpServer, userName, passWord)
        em_obj.add_text('这是发送到qq邮箱的测试邮件发送代码，无需回复')
        em_obj.send('xxx@qq.com')
        em_obj.add_text('这是发送到gmail测试邮件发送代码，无需回复')
        em_obj.sends(['xxx@gmail.com'])
        em_obj.quit()
        print("发送成功")

    def testSendEmailImage(self):
        em_obj = EmailSender(smtpServer, userName, passWord)
        em_obj.add_text('随带图片测试邮件，无需回复')
        em_obj.add_img("baidulogo.png", 'images/baidulogo.png')
        em_obj.send('xxx@qq.com')
        em_obj.quit()
        print("发送成功")

    def testSendEmailFile(self):
        em_obj = EmailSender(smtpServer, userName, passWord)
        em_obj.add_text('随带文件的测试邮件，无需回复')
        em_obj.add_file("testddd.zip", 'images/testddd.zip')
        em_obj.send('xxx@qq.com')
        em_obj.quit()
        print("发送成功")